@@ -18,8 +18,6 @@ Routines for configuring Glance
 """
 
 import logging
-import logging.config
-import logging.handlers
 import os
 
 from oslo_config import cfg
@@ -94,7 +92,7 @@ image_format_opts = [
                                                    group='DEFAULT')]),
     cfg.ListOpt('disk_formats',
                 default=['ami', 'ari', 'aki', 'vhd', 'vhdx', 'vmdk', 'raw',
-                         'qcow2', 'vdi', 'iso'],
+                         'qcow2', 'vdi', 'iso', 'ploop'],
                 help=_("Supported values for the 'disk_format' "
                        "image attribute"),
                 deprecated_opts=[cfg.DeprecatedOpt('disk_formats',
@@ -193,6 +191,40 @@ Possible values:
 Related options:
     * image_property_quota
 
+""")),
+    cfg.StrOpt('hashing_algorithm',
+               default='sha512',
+               help=_(""""
+Secure hashing algorithm used for computing the 'os_hash_value' property.
+
+This option configures the Glance "multihash", which consists of two
+image properties: the 'os_hash_algo' and the 'os_hash_value'.  The
+'os_hash_algo' will be populated by the value of this configuration
+option, and the 'os_hash_value' will be populated by the hexdigest computed
+when the algorithm is applied to the uploaded or imported image data.
+
+The value must be a valid secure hash algorithm name recognized by the
+python 'hashlib' library.  You can determine what these are by examining
+the 'hashlib.algorithms_available' data member of the version of the
+library being used in your Glance installation.  For interoperability
+purposes, however, we recommend that you use the set of secure hash
+names supplied by the 'hashlib.algorithms_guaranteed' data member because
+those algorithms are guaranteed to be supported by the 'hashlib' library
+on all platforms.  Thus, any image consumer using 'hashlib' locally should
+be able to verify the 'os_hash_value' of the image.
+
+The default value of 'sha512' is a performant secure hash algorithm.
+
+If this option is misconfigured, any attempts to store image data will fail.
+For that reason, we recommend using the default value.
+
+Possible values:
+    * Any secure hash algorithm name recognized by the Python 'hashlib'
+      library
+
+Related options:
+    * None
+
 """)),
     cfg.IntOpt('image_member_quota', default=128,
                help=_("""
@@ -251,6 +283,14 @@ Related options:
     # glance/tests/functional/v2/test_images.py
     cfg.StrOpt('data_api',
                default='glance.db.sqlalchemy.api',
+                       deprecated_for_removal=True,
+                       deprecated_since="Queens",
+                       deprecated_reason=_("""
+Glance registry service is deprecated for removal.
+
+More information can be found from the spec:
+http://specs.openstack.org/openstack/glance-specs/specs/queens/approved/glance/deprecate-registry.html
+"""),
                help=_("""
 Python module path of data access API.
 
@@ -367,10 +407,11 @@ Related options:
     # related rules set to admin only once this option is finally removed.
     cfg.BoolOpt('show_multiple_locations', default=False,
                 deprecated_for_removal=True,
-                deprecated_reason=_('This option will be removed in the Ocata '
-                                    'release because the same functionality '
-                                    'can be achieved with greater granularity '
-                                    'by using policies. Please see the Newton '
+                deprecated_reason=_('This option will be removed in the Pike '
+                                    'release or later because the same '
+                                    'functionality can be achieved with '
+                                    'greater granularity by using policies. '
+                                    'Please see the Newton '
                                     'release notes for more information.'),
                 deprecated_since='Newton',
                 help=_("""
@@ -447,50 +488,6 @@ Possible values:
 Related options:
     * None
 
-""")),
-    # NOTE(nikhil): Even though deprecated, the configuration option
-    # ``enable_v1_api`` is set to True by default on purpose. Having it enabled
-    # helps the projects that haven't been able to fully move to v2 yet by
-    # keeping the devstack setup to use glance v1 as well. We need to switch it
-    # to False by default soon after Newton is cut so that we can identify the
-    # projects that haven't moved to v2 yet and start having some interesting
-    # conversations with them. Switching to False in Newton may result into
-    # destabilizing the gate and affect the release.
-    cfg.BoolOpt('enable_v1_api',
-                default=True,
-                deprecated_reason=_DEPRECATE_GLANCE_V1_MSG,
-                deprecated_since='Newton',
-                help=_("""
-Deploy the v1 OpenStack Images API.
-
-When this option is set to ``True``, Glance service will respond to
-requests on registered endpoints conforming to the v1 OpenStack
-Images API.
-
-NOTES:
-    * If this option is enabled, then ``enable_v1_registry`` must
-      also be set to ``True`` to enable mandatory usage of Registry
-      service with v1 API.
-
-    * If this option is disabled, then the ``enable_v1_registry``
-      option, which is enabled by default, is also recommended
-      to be disabled.
-
-    * This option is separate from ``enable_v2_api``, both v1 and v2
-      OpenStack Images API can be deployed independent of each
-      other.
-
-    * If deploying only the v2 Images API, this option, which is
-      enabled by default, should be disabled.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * enable_v1_registry
-    * enable_v2_api
-
 """)),
     cfg.BoolOpt('enable_v2_api',
                 default=True,
@@ -516,20 +513,12 @@ NOTES:
       option, which is enabled by default, is also recommended
       to be disabled.
 
-    * This option is separate from ``enable_v1_api``, both v1 and v2
-      OpenStack Images API can be deployed independent of each
-      other.
-
-    * If deploying only the v1 Images API, this option, which is
-      enabled by default, should be disabled.
-
 Possible values:
     * True
     * False
 
 Related options:
     * enable_v2_registry
-    * enable_v1_api
 
 """)),
     cfg.BoolOpt('enable_v1_registry',
@@ -537,28 +526,18 @@ Related options:
                 deprecated_reason=_DEPRECATE_GLANCE_V1_MSG,
                 deprecated_since='Newton',
                 help=_("""
-Deploy the v1 API Registry service.
-
-When this option is set to ``True``, the Registry service
-will be enabled in Glance for v1 API requests.
-
-NOTES:
-    * Use of Registry is mandatory in v1 API, so this option must
-      be set to ``True`` if the ``enable_v1_api`` option is enabled.
-
-    * If deploying only the v2 OpenStack Images API, this option,
-      which is enabled by default, should be disabled.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * enable_v1_api
-
+                    DEPRECATED FOR REMOVAL
 """)),
     cfg.BoolOpt('enable_v2_registry',
                 default=True,
+                deprecated_for_removal=True,
+                deprecated_since="Queens",
+                deprecated_reason=_("""
+Glance registry service is deprecated for removal.
+
+More information can be found from the spec:
+http://specs.openstack.org/openstack/glance-specs/specs/queens/approved/glance/deprecate-registry.html
+"""),
                 help=_("""
 Deploy the v2 API Registry service.
 
@@ -583,9 +562,9 @@ Related options:
     * data_api
 
 """)),
-    cfg.StrOpt('pydev_worker_debug_host',
-               sample_default='localhost',
-               help=_("""
+    cfg.HostAddressOpt('pydev_worker_debug_host',
+                       sample_default='localhost',
+                       help=_("""
 Host address of the pydev server.
 
 Provide a string value representing the hostname or IP of the
@@ -661,6 +640,43 @@ Relation options:
     * None
 
 """)),
+    cfg.StrOpt('node_staging_uri',
+               default='file:///tmp/staging/',
+               help=_("""
+The URL provides location where the temporary data will be stored
+
+This option is for Glance internal use only. Glance will save the
+image data uploaded by the user to 'staging' endpoint during the
+image import process.
+
+This option does not change the 'staging' API endpoint by any means.
+
+NOTE: It is discouraged to use same path as [task]/work_dir
+
+NOTE: 'file://<absolute-directory-path>' is the only option
+api_image_import flow will support for now.
+
+NOTE: The staging path must be on shared filesystem available to all
+Glance API nodes.
+
+Possible values:
+    * String starting with 'file://' followed by absolute FS path
+
+Related options:
+    * [task]/work_dir
+
+""")),
+    cfg.ListOpt('enabled_import_methods',
+                item_type=cfg.types.String(quotes=True),
+                bounds=True,
+                default=['glance-direct', 'web-download'],
+                help=_("""
+List of enabled Image Import Methods
+
+Both 'glance-direct' and 'web-download' are enabled by default.
+
+Related options:
+    * [DEFAULT]/node_staging_uri""")),
 ]
 
 CONF = cfg.CONF
@@ -717,7 +733,7 @@ def _get_deployment_config_file():
     path = CONF.paste_deploy.config_file
     if not path:
         path = _get_paste_config_path()
-    if not path:
+    if not path or not (os.path.isfile(os.path.abspath(path))):
         msg = _("Unable to locate paste config file for %s.") % CONF.prog
         raise RuntimeError(msg)
     return os.path.abspath(path)
@@ -734,7 +750,7 @@ def load_paste_app(app_name, flavor=None, conf_file=None):
     :param flavor: name of the variant of the application to load
     :param conf_file: path to the paste config file
 
-    :raises: RuntimeError when config file cannot be located or application
+    :raises RuntimeError: when config file cannot be located or application
             cannot be loaded from config file
     """
     # append the deployment flavor to the application name,
@@ -773,28 +789,26 @@ def set_config_defaults():
 
 def set_cors_middleware_defaults():
     """Update default configuration options for oslo.middleware."""
-    # CORS Defaults
-    # TODO(krotscheck): Update with https://review.openstack.org/#/c/285368/
-    cfg.set_defaults(cors.CORS_OPTS,
-                     allow_headers=['Content-MD5',
-                                    'X-Image-Meta-Checksum',
-                                    'X-Storage-Token',
-                                    'Accept-Encoding',
-                                    'X-Auth-Token',
-                                    'X-Identity-Status',
-                                    'X-Roles',
-                                    'X-Service-Catalog',
-                                    'X-User-Id',
-                                    'X-Tenant-Id',
-                                    'X-OpenStack-Request-ID'],
-                     expose_headers=['X-Image-Meta-Checksum',
-                                     'X-Auth-Token',
-                                     'X-Subject-Token',
-                                     'X-Service-Token',
-                                     'X-OpenStack-Request-ID'],
-                     allow_methods=['GET',
-                                    'PUT',
-                                    'POST',
-                                    'DELETE',
-                                    'PATCH']
-                     )
+    cors.set_defaults(
+        allow_headers=['Content-MD5',
+                       'X-Image-Meta-Checksum',
+                       'X-Storage-Token',
+                       'Accept-Encoding',
+                       'X-Auth-Token',
+                       'X-Identity-Status',
+                       'X-Roles',
+                       'X-Service-Catalog',
+                       'X-User-Id',
+                       'X-Tenant-Id',
+                       'X-OpenStack-Request-ID'],
+        expose_headers=['X-Image-Meta-Checksum',
+                        'X-Auth-Token',
+                        'X-Subject-Token',
+                        'X-Service-Token',
+                        'X-OpenStack-Request-ID'],
+        allow_methods=['GET',
+                       'PUT',
+                       'POST',
+                       'DELETE',
+                       'PATCH']
+    )
