@@ -28,7 +28,6 @@ import signal
 import sys
 import time
 
-import eventlet
 from eventlet.green import socket
 from eventlet.green import ssl
 import eventlet.greenio
@@ -41,7 +40,6 @@ from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 from oslo_utils import strutils
 from osprofiler import opts as profiler_opts
-import routes
 import routes.middleware
 import six
 import webob.dec
@@ -55,17 +53,16 @@ from glance import i18n
 from glance.i18n import _, _LE, _LI, _LW
 
 
+try:
+    from webob.acceptparse import AcceptLanguageValidHeader  # noqa
+    USING_WEBOB_1_8 = True
+except ImportError:
+    USING_WEBOB_1_8 = False
+
 bind_opts = [
-    # NOTE(dharinic): Since ``bind_host`` accepts symbolic names
-    # mapped to IP address apart from IP addresses, we use
-    # ``StrOpt`` as against ``IPOpt`` for backward compatibility
-    # reasons. We do not use ``HostnameOpt`` as it would prevent the
-    # opt from accepting IPv6 addresses.
-    # TODO(dharinic): Change the Opt type upon resolution of
-    # bug-1619044
-    cfg.StrOpt('bind_host',
-               default='0.0.0.0',
-               help=_("""
+    cfg.HostAddressOpt('bind_host',
+                       default='0.0.0.0',
+                       help=_("""
 IP address to bind the glance servers to.
 
 Provide an IP address to bind the glance server to. The default
@@ -216,7 +213,10 @@ Number of Glance worker processes to start.
 
 Provide a non-negative integer value to set the number of child
 process workers to service requests. By default, the number of CPUs
-available is set as the value for ``workers``.
+available is set as the value for ``workers`` limited to 8. For
+example if the processor count is 6, 6 workers will be used, if the
+processor count is 24 only 8 workers will be used. The limit will only
+apply to the default value, if 24 workers is configured, 24 is used.
 
 Each worker process is made to listen on the port set in the
 configuration file and contains a greenthread pool of size 1000.
@@ -317,6 +317,13 @@ wsgi_opts = [
                       '"HTTP_X_FORWARDED_PROTO".')),
 ]
 
+store_opts = [
+    cfg.DictOpt('enabled_backends',
+                help=_('Key:Value pair of store identifier and store type. '
+                       'In case of multiple backends should be separated'
+                       'using comma.')),
+]
+
 
 LOG = logging.getLogger(__name__)
 
@@ -325,16 +332,27 @@ CONF.register_opts(bind_opts)
 CONF.register_opts(socket_opts)
 CONF.register_opts(eventlet_opts)
 CONF.register_opts(wsgi_opts)
+CONF.register_opts(store_opts)
 profiler_opts.set_defaults(CONF)
 
 ASYNC_EVENTLET_THREAD_POOL_LIST = []
 
+# Detect if we're running under the uwsgi server
+try:
+    import uwsgi
+    LOG.debug('Detected running under uwsgi')
+except ImportError:
+    LOG.debug('Detected not running under uwsgi')
+    uwsgi = None
+
 
 def get_num_workers():
     """Return the configured number of workers."""
     if CONF.workers is None:
-        # None implies the number of CPUs
-        return processutils.get_worker_count()
+        # None implies the number of CPUs limited to 8
+        # See Launchpad bug #1748916 and the config help text
+        workers = processutils.get_worker_count()
+        return workers if workers < 8 else 8
     return CONF.workers
 
 
@@ -438,6 +456,13 @@ def initialize_glance_store():
     glance_store.verify_default_store()
 
 
+def initialize_multi_store():
+    """Initialize glance multi store backends."""
+    glance_store.register_store_opts(CONF)
+    glance_store.create_multi_stores(CONF)
+    glance_store.verify_store()
+
+
 def get_asynchronous_eventlet_pool(size=1000):
     """Return eventlet pool to caller.
 
@@ -500,6 +525,7 @@ class Server(object):
         """Kills the entire process group."""
         signal.signal(signal.SIGTERM, signal.SIG_IGN)
         signal.signal(signal.SIGINT, signal.SIG_IGN)
+        signal.signal(signal.SIGCHLD, signal.SIG_IGN)
         self.running = False
         os.killpg(self.pgid, signal.SIGTERM)
 
@@ -588,7 +614,10 @@ class Server(object):
         self.client_socket_timeout = CONF.client_socket_timeout or None
         self.configure_socket(old_conf, has_changed)
         if self.initialize_glance_store:
-            initialize_glance_store()
+            if CONF.enabled_backends:
+                initialize_multi_store()
+            else:
+                initialize_glance_store()
 
     def reload(self):
         """
@@ -933,6 +962,31 @@ class Router(object):
         return app
 
 
+class _UWSGIChunkFile(object):
+
+    def read(self, length=None):
+        position = 0
+        if length == 0:
+            return b""
+
+        if length and length < 0:
+            length = None
+
+        response = []
+        while True:
+            data = uwsgi.chunked_read()
+            # Return everything if we reached the end of the file
+            if not data:
+                break
+            response.append(data)
+            # Return the data if we've reached the length
+            if length is not None:
+                position += len(data)
+                if position >= length:
+                    break
+        return b''.join(response)
+
+
 class Request(webob.Request):
     """Add some OpenStack API-specific logic to the base webob.Request."""
 
@@ -943,6 +997,39 @@ class Request(webob.Request):
                 environ['wsgi.url_scheme'] = scheme
         super(Request, self).__init__(environ, *args, **kwargs)
 
+    @property
+    def body_file(self):
+        if uwsgi:
+            if self.headers.get('transfer-encoding', '').lower() == 'chunked':
+                return _UWSGIChunkFile()
+        return super(Request, self).body_file
+
+    @body_file.setter
+    def body_file(self, value):
+        # NOTE(cdent): If you have a property setter in a superclass, it will
+        # not be inherited.
+        webob.Request.body_file.fset(self, value)
+
+    @property
+    def params(self):
+        """Override params property of webob.request.BaseRequest.
+
+        Added an 'encoded_params' attribute in case of PY2 to avoid
+        encoding values in next subsequent calls to the params property.
+        """
+        if six.PY2:
+            encoded_params = getattr(self, 'encoded_params', None)
+            if encoded_params is None:
+                params = super(Request, self).params
+                params_dict = multidict.MultiDict()
+                for key, value in params.items():
+                    params_dict.add(key, encodeutils.safe_encode(value))
+
+                setattr(self, 'encoded_params',
+                        multidict.NestedMultiDict(params_dict))
+            return self.encoded_params
+        return super(Request, self).params
+
     def best_match_content_type(self):
         """Determine the requested response content-type."""
         supported = ('application/json',)
@@ -961,7 +1048,7 @@ class Request(webob.Request):
         else:
             return content_type
 
-    def best_match_language(self):
+    def _best_match_language_1_7(self):
         """Determines best available locale from the Accept-Language header.
 
         :returns: the best language match or None if the 'Accept-Language'
@@ -972,20 +1059,93 @@ class Request(webob.Request):
         langs = i18n.get_available_languages('glance')
         return self.accept_language.best_match(langs)
 
-    def get_content_range(self):
+    def _best_match_language_1_8(self):
+        """Determines best available locale from the Accept-Language header.
+
+        :returns: the best language match or None if the 'Accept-Language'
+                  header was not available in the request.
+        """
+        if not self.accept_language:
+            return None
+        langs = i18n.get_available_languages('glance')
+        # NOTE(rosmaita): give the webob lookup() function a sentinal value
+        # for default so we can preserve the behavior of this function as
+        # indicated by the current unit tests.  See Launchpad bug #1765748.
+        best_match = self.accept_language.lookup(langs, default='fake_LANG')
+        if best_match == 'fake_LANG':
+            best_match = None
+        return best_match
+
+    def best_match_language(self):
+        if USING_WEBOB_1_8:
+            return self._best_match_language_1_8()
+        else:
+            return self._best_match_language_1_7()
+
+    def get_range_from_request(self, image_size):
         """Return the `Range` in a request."""
-        range_str = self.headers.get('Content-Range')
+
+        range_str = self.headers.get('Range')
         if range_str is not None:
-            range_ = webob.byterange.ContentRange.parse(range_str)
-            if range_ is None:
-                msg = _('Malformed Content-Range header: %s') % range_str
+
+            # NOTE(dharinic): We do not support multi range requests.
+            if ',' in range_str:
+                msg = ("Requests with multiple ranges are not supported in "
+                       "Glance. You may make multiple single-range requests "
+                       "instead.")
                 raise webob.exc.HTTPBadRequest(explanation=msg)
+
+            range_ = webob.byterange.Range.parse(range_str)
+            if range_ is None:
+                msg = ("Invalid Range header.")
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
+            # NOTE(dharinic): Ensure that a range like bytes=4- for an image
+            # size of 3 is invalidated as per rfc7233.
+            if range_.start >= image_size:
+                msg = ("Invalid start position in Range header. "
+                       "Start position MUST be in the inclusive range [0, %s]."
+                       % (image_size - 1))
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
             return range_
 
+        # NOTE(dharinic): For backward compatibility reasons, we maintain
+        # support for 'Content-Range' in requests even though it's not
+        # correct to use it in requests..
+        c_range_str = self.headers.get('Content-Range')
+        if c_range_str is not None:
+            content_range = webob.byterange.ContentRange.parse(c_range_str)
+            # NOTE(dharinic): Ensure that a content range like 1-4/* for an
+            # image size of 3 is invalidated.
+            if content_range is None:
+                msg = ("Invalid Content-Range header.")
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
+            if (content_range.length is None and
+                    content_range.stop > image_size):
+                msg = ("Invalid stop position in Content-Range header. "
+                       "The stop position MUST be in the inclusive range "
+                       "[0, %s]." % (image_size - 1))
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
+            if content_range.start >= image_size:
+                msg = ("Invalid start position in Content-Range header. "
+                       "Start position MUST be in the inclusive range [0, %s]."
+                       % (image_size - 1))
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
+            return content_range
+
 
 class JSONRequestDeserializer(object):
     valid_transfer_encoding = frozenset(['chunked', 'compress', 'deflate',
                                          'gzip', 'identity'])
+    httpverb_may_have_body = frozenset({'POST', 'PUT', 'PATCH'})
+
+    @classmethod
+    def is_valid_encoding(cls, request):
+        request_encoding = request.headers.get('transfer-encoding', '').lower()
+        return request_encoding in cls.valid_transfer_encoding
+
+    @classmethod
+    def is_valid_method(cls, request):
+        return request.method.upper() in cls.httpverb_may_have_body
 
     def has_body(self, request):
         """
@@ -993,11 +1153,12 @@ class JSONRequestDeserializer(object):
 
         :param request:  Webob.Request object
         """
-        request_encoding = request.headers.get('transfer-encoding', '').lower()
-        is_valid_encoding = request_encoding in self.valid_transfer_encoding
-        if is_valid_encoding and request.is_body_readable:
+
+        if self.is_valid_encoding(request) and self.is_valid_method(request):
+            request.is_body_readable = True
             return True
-        elif request.content_length is not None and request.content_length > 0:
+
+        if request.content_length is not None and request.content_length > 0:
             return True
 
         return False
@@ -1126,6 +1287,10 @@ class Resource(object):
             response = webob.exc.HTTPInternalServerError()
             return response
 
+        # We cannot serialize an Exception, so return the action_result
+        if isinstance(action_result, Exception):
+            return action_result
+
         try:
             response = webob.Response(request=request)
             self.dispatch(self.serializer, action, response, action_result)
