@@ -42,7 +42,7 @@ class FakeImage(object):
     locations = [{'url': 'file:///not/a/path', 'metadata': {}}]
     tags = set([])
 
-    def set_data(self, data, size=None):
+    def set_data(self, data, size=None, backend=None):
         self.size = 0
         for d in data:
             self.size += len(d)
@@ -55,9 +55,6 @@ class TestImageQuota(test_utils.BaseTestCase):
     def setUp(self):
         super(TestImageQuota, self).setUp()
 
-    def tearDown(self):
-        super(TestImageQuota, self).tearDown()
-
     def _get_image(self, location_count=1, image_size=10):
         context = FakeContext()
         db_api = unit_test_utils.FakeDB()
