@@ -16,8 +16,10 @@ import uuid
 
 from cursive import exception as cursive_exception
 import glance_store
+from glance_store._drivers import filesystem
 import mock
 import six
+from six.moves import http_client as http
 import webob
 
 import glance.api.policy
@@ -62,10 +64,12 @@ class FakeImage(object):
         else:
             self._status = value
 
-    def get_data(self, *args, **kwargs):
-        return self.data
+    def get_data(self, offset=0, chunk_size=None):
+        if chunk_size:
+            return self.data[offset:offset + chunk_size]
+        return self.data[offset:]
 
-    def set_data(self, data, size=None):
+    def set_data(self, data, size=None, backend=None):
         self.data = ''.join(data)
         self.size = size
         self.status = 'modified-by-fake'
@@ -86,7 +90,12 @@ class FakeImageRepo(object):
 
 
 class FakeGateway(object):
-    def __init__(self, repo):
+    def __init__(self, db=None, store=None, notifier=None,
+                 policy=None, repo=None):
+        self.db = db
+        self.store = store
+        self.notifier = notifier
+        self.policy = policy
         self.repo = repo
 
     def get_repo(self, context):
@@ -99,9 +108,13 @@ class TestImagesController(base.StoreClearingUnitTest):
 
         self.config(debug=True)
         self.image_repo = FakeImageRepo()
-        self.gateway = FakeGateway(self.image_repo)
-        self.controller = glance.api.v2.image_data.ImageDataController(
-            gateway=self.gateway)
+        db = unit_test_utils.FakeDB()
+        policy = unit_test_utils.FakePolicyEnforcer()
+        notifier = unit_test_utils.FakeNotifier()
+        store = unit_test_utils.FakeStoreAPI()
+        self.controller = glance.api.v2.image_data.ImageDataController()
+        self.controller.gateway = FakeGateway(db, store, notifier, policy,
+                                              self.image_repo)
 
     def test_download(self):
         request = unit_test_utils.get_fake_request()
@@ -187,6 +200,16 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertEqual('YYYY', image.data)
         self.assertIsNone(image.size)
 
+    @mock.patch.object(glance.api.policy.Enforcer, 'enforce')
+    def test_upload_image_forbidden(self, mock_enforce):
+        request = unit_test_utils.get_fake_request()
+        mock_enforce.side_effect = exception.Forbidden
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.upload,
+                          request, unit_test_utils.UUID2, 'YYYY', 4)
+        mock_enforce.assert_called_once_with(request.context,
+                                             "upload_image",
+                                             {})
+
     def test_upload_invalid(self):
         request = unit_test_utils.get_fake_request()
         image = FakeImage('abcd')
@@ -457,6 +480,126 @@ class TestImagesController(base.StoreClearingUnitTest):
                           request, unit_test_utils.UUID2, 'ZZZ', 3)
         self.assertEqual('queued', self.image_repo.saved_image.status)
 
+    @mock.patch.object(filesystem.Store, 'add')
+    def test_restore_image_when_staging_failed(self, mock_store_add):
+        mock_store_add.side_effect = glance_store.StorageWriteDenied()
+        request = unit_test_utils.get_fake_request()
+        image_id = str(uuid.uuid4())
+        image = FakeImage('fake')
+        self.image_repo.result = image
+        self.assertRaises(webob.exc.HTTPServiceUnavailable,
+                          self.controller.stage,
+                          request, image_id, 'YYYYYYY', 7)
+        self.assertEqual('queued', self.image_repo.saved_image.status)
+
+    def test_stage(self):
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        with mock.patch.object(filesystem.Store, 'add'):
+            self.controller.stage(request, image_id, 'YYYY', 4)
+        self.assertEqual('uploading', image.status)
+        self.assertEqual(0, image.size)
+
+    def test_image_already_on_staging(self):
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        with mock.patch.object(filesystem.Store, 'add') as mock_store_add:
+            self.controller.stage(request, image_id, 'YYYY', 4)
+            self.assertEqual('uploading', image.status)
+            mock_store_add.side_effect = glance_store.Duplicate()
+            self.assertEqual(0, image.size)
+            self.assertRaises(webob.exc.HTTPConflict, self.controller.stage,
+                              request, image_id, 'YYYY', 4)
+
+    @mock.patch.object(glance_store.driver.Store, 'configure')
+    def test_image_stage_raises_bad_store_uri(self, mock_store_configure):
+        mock_store_configure.side_effect = AttributeError()
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        self.assertRaises(exception.BadStoreUri, self.controller.stage,
+                          request, image_id, 'YYYY', 4)
+
+    @mock.patch.object(filesystem.Store, 'add')
+    def test_image_stage_raises_storage_full(self, mock_store_add):
+        mock_store_add.side_effect = glance_store.StorageFull()
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        with mock.patch.object(self.controller, "_unstage"):
+            self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
+                              self.controller.stage,
+                              request, image_id, 'YYYYYYY', 7)
+
+    @mock.patch.object(filesystem.Store, 'add')
+    def test_image_stage_raises_storage_quota_full(self, mock_store_add):
+        mock_store_add.side_effect = exception.StorageQuotaFull("message")
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        with mock.patch.object(self.controller, "_unstage"):
+            self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
+                              self.controller.stage,
+                              request, image_id, 'YYYYYYY', 7)
+
+    @mock.patch.object(filesystem.Store, 'add')
+    def test_image_stage_raises_storage_write_denied(self, mock_store_add):
+        mock_store_add.side_effect = glance_store.StorageWriteDenied()
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        with mock.patch.object(self.controller, "_unstage"):
+            self.assertRaises(webob.exc.HTTPServiceUnavailable,
+                              self.controller.stage,
+                              request, image_id, 'YYYYYYY', 7)
+
+    def test_image_stage_raises_internal_error(self):
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        self.image_repo.result = exception.ServerError()
+        self.assertRaises(exception.ServerError,
+                          self.controller.stage,
+                          request, image_id, 'YYYYYYY', 7)
+
+    def test_image_stage_non_existent_image(self):
+        request = unit_test_utils.get_fake_request()
+        self.image_repo.result = exception.NotFound()
+        self.assertRaises(webob.exc.HTTPNotFound, self.controller.stage,
+                          request, str(uuid.uuid4()), 'ABC', 3)
+
+    @mock.patch.object(filesystem.Store, 'add')
+    def test_image_stage_raises_image_size_exceeded(self, mock_store_add):
+        mock_store_add.side_effect = exception.ImageSizeLimitExceeded()
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        with mock.patch.object(self.controller, "_unstage"):
+            self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
+                              self.controller.stage,
+                              request, image_id, 'YYYYYYY', 7)
+
+    @mock.patch.object(filesystem.Store, 'add')
+    def test_image_stage_invalid_image_transition(self, mock_store_add):
+        image_id = str(uuid.uuid4())
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage(image_id=image_id)
+        self.image_repo.result = image
+        self.controller.stage(request, image_id, 'YYYY', 4)
+        self.assertEqual('uploading', image.status)
+        self.assertEqual(0, image.size)
+        # try staging again
+        mock_store_add.side_effect = exception.InvalidImageStatusTransition(
+            cur_status='uploading', new_status='uploading')
+        self.assertRaises(webob.exc.HTTPConflict, self.controller.stage,
+                          request, image_id, 'YYYY', 4)
+
 
 class TestImageDataDeserializer(test_utils.BaseTestCase):
 
@@ -527,6 +670,31 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         self.assertRaises(webob.exc.HTTPUnsupportedMediaType,
                           self.deserializer.upload, request)
 
+    def test_stage(self):
+        req = unit_test_utils.get_fake_request()
+        req.headers['Content-Type'] = 'application/octet-stream'
+        req.headers['Content-Length'] = 4
+        req.body_file = six.BytesIO(b'YYYY')
+        output = self.deserializer.stage(req)
+        data = output.pop('data')
+        self.assertEqual(b'YYYY', data.read())
+
+    def test_stage_without_glance_direct(self):
+        self.config(enabled_import_methods=['web-download'])
+        req = unit_test_utils.get_fake_request()
+        self.assertRaises(webob.exc.HTTPNotFound,
+                          self.deserializer.stage,
+                          req)
+
+    def test_stage_raises_invalid_content_type(self):
+        # TODO(abhishekk): change this when import methods are
+        # listed in the config file
+        req = unit_test_utils.get_fake_request()
+        req.headers['Content-Type'] = 'application/json'
+        self.assertRaises(webob.exc.HTTPUnsupportedMediaType,
+                          self.deserializer.stage,
+                          req)
+
 
 class TestImageDataSerializer(test_utils.BaseTestCase):
 
@@ -547,6 +715,158 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         self.assertEqual('application/octet-stream',
                          response.headers['Content-Type'])
 
+    def test_range_requests_for_image_downloads(self):
+        """
+        Test partial download 'Range' requests for images (random image access)
+        """
+        def download_successful_Range(d_range):
+            request = wsgi.Request.blank('/')
+            request.environ = {}
+            request.headers['Range'] = d_range
+            response = webob.Response()
+            response.request = request
+            image = FakeImage(size=3, data=[b'X', b'Y', b'Z'])
+            self.serializer.download(response, image)
+            self.assertEqual(206, response.status_code)
+            self.assertEqual('2', response.headers['Content-Length'])
+            self.assertEqual('bytes 1-2/3', response.headers['Content-Range'])
+            self.assertEqual(b'YZ', response.body)
+
+        download_successful_Range('bytes=1-2')
+        download_successful_Range('bytes=1-')
+        download_successful_Range('bytes=1-3')
+        download_successful_Range('bytes=-2')
+        download_successful_Range('bytes=1-100')
+
+        def full_image_download_w_range(d_range):
+            request = wsgi.Request.blank('/')
+            request.environ = {}
+            request.headers['Range'] = d_range
+            response = webob.Response()
+            response.request = request
+            image = FakeImage(size=3, data=[b'X', b'Y', b'Z'])
+            self.serializer.download(response, image)
+            self.assertEqual(206, response.status_code)
+            self.assertEqual('3', response.headers['Content-Length'])
+            self.assertEqual('bytes 0-2/3', response.headers['Content-Range'])
+            self.assertEqual(b'XYZ', response.body)
+
+        full_image_download_w_range('bytes=0-')
+        full_image_download_w_range('bytes=0-2')
+        full_image_download_w_range('bytes=0-3')
+        full_image_download_w_range('bytes=-3')
+        full_image_download_w_range('bytes=-4')
+        full_image_download_w_range('bytes=0-100')
+        full_image_download_w_range('bytes=-100')
+
+        def download_failures_Range(d_range):
+            request = wsgi.Request.blank('/')
+            request.environ = {}
+            request.headers['Range'] = d_range
+            response = webob.Response()
+            response.request = request
+            image = FakeImage(size=3, data=[b'Z', b'Z', b'Z'])
+            self.assertRaises(webob.exc.HTTPRequestRangeNotSatisfiable,
+                              self.serializer.download,
+                              response, image)
+            return
+
+        download_failures_Range('bytes=4-1')
+        download_failures_Range('bytes=4-')
+        download_failures_Range('bytes=3-')
+        download_failures_Range('bytes=1')
+        download_failures_Range('bytes=100')
+        download_failures_Range('bytes=100-')
+        download_failures_Range('bytes=')
+
+    def test_multi_range_requests_raises_bad_request_error(self):
+        request = wsgi.Request.blank('/')
+        request.environ = {}
+        request.headers['Range'] = 'bytes=0-0,-1'
+        response = webob.Response()
+        response.request = request
+        image = FakeImage(size=3, data=[b'Z', b'Z', b'Z'])
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.serializer.download,
+                          response, image)
+
+    def test_download_failure_with_valid_range(self):
+        with mock.patch.object(glance.api.policy.ImageProxy,
+                               'get_data') as mock_get_data:
+            mock_get_data.side_effect = glance_store.NotFound(image="image")
+        request = wsgi.Request.blank('/')
+        request.environ = {}
+        request.headers['Range'] = 'bytes=1-2'
+        response = webob.Response()
+        response.request = request
+        image = FakeImage(size=3, data=[b'Z', b'Z', b'Z'])
+        image.get_data = mock_get_data
+        self.assertRaises(webob.exc.HTTPNoContent,
+                          self.serializer.download,
+                          response, image)
+
+    def test_content_range_requests_for_image_downloads(self):
+        """
+        Even though Content-Range is incorrect on requests, we support it
+        for backward compatibility with clients written for pre-Pike
+        Glance.
+        The following test is for 'Content-Range' requests, which we have
+        to ensure that we prevent regression.
+        """
+        def download_successful_ContentRange(d_range):
+            request = wsgi.Request.blank('/')
+            request.environ = {}
+            request.headers['Content-Range'] = d_range
+            response = webob.Response()
+            response.request = request
+            image = FakeImage(size=3, data=[b'X', b'Y', b'Z'])
+            self.serializer.download(response, image)
+            self.assertEqual(206, response.status_code)
+            self.assertEqual('2', response.headers['Content-Length'])
+            self.assertEqual('bytes 1-2/3', response.headers['Content-Range'])
+            self.assertEqual(b'YZ', response.body)
+
+        download_successful_ContentRange('bytes 1-2/3')
+        download_successful_ContentRange('bytes 1-2/*')
+
+        def download_failures_ContentRange(d_range):
+            request = wsgi.Request.blank('/')
+            request.environ = {}
+            request.headers['Content-Range'] = d_range
+            response = webob.Response()
+            response.request = request
+            image = FakeImage(size=3, data=[b'Z', b'Z', b'Z'])
+            self.assertRaises(webob.exc.HTTPRequestRangeNotSatisfiable,
+                              self.serializer.download,
+                              response, image)
+            return
+
+        download_failures_ContentRange('bytes -3/3')
+        download_failures_ContentRange('bytes 1-/3')
+        download_failures_ContentRange('bytes 1-3/3')
+        download_failures_ContentRange('bytes 1-4/3')
+        download_failures_ContentRange('bytes 1-4/*')
+        download_failures_ContentRange('bytes 4-1/3')
+        download_failures_ContentRange('bytes 4-1/*')
+        download_failures_ContentRange('bytes 4-8/*')
+        download_failures_ContentRange('bytes 4-8/10')
+        download_failures_ContentRange('bytes 4-8/3')
+
+    def test_download_failure_with_valid_content_range(self):
+        with mock.patch.object(glance.api.policy.ImageProxy,
+                               'get_data') as mock_get_data:
+            mock_get_data.side_effect = glance_store.NotFound(image="image")
+        request = wsgi.Request.blank('/')
+        request.environ = {}
+        request.headers['Content-Range'] = 'bytes %s-%s/3' % (1, 2)
+        response = webob.Response()
+        response.request = request
+        image = FakeImage(size=3, data=[b'Z', b'Z', b'Z'])
+        image.get_data = mock_get_data
+        self.assertRaises(webob.exc.HTTPNoContent,
+                          self.serializer.download,
+                          response, image)
+
     def test_download_with_checksum(self):
         request = wsgi.Request.blank('/')
         request.environ = {}
@@ -662,5 +982,46 @@ class TestImageDataSerializer(test_utils.BaseTestCase):
         response = webob.Response()
         response.request = request
         self.serializer.upload(response, {})
-        self.assertEqual(204, response.status_int)
+        self.assertEqual(http.NO_CONTENT, response.status_int)
         self.assertEqual('0', response.headers['Content-Length'])
+
+    def test_stage(self):
+        request = webob.Request.blank('/')
+        request.environ = {}
+        response = webob.Response()
+        response.request = request
+        self.serializer.stage(response, {})
+        self.assertEqual(http.NO_CONTENT, response.status_int)
+        self.assertEqual('0', response.headers['Content-Length'])
+
+
+class TestMultiBackendImagesController(base.MultiStoreClearingUnitTest):
+
+    def setUp(self):
+        super(TestMultiBackendImagesController, self).setUp()
+
+        self.config(debug=True)
+        self.image_repo = FakeImageRepo()
+        db = unit_test_utils.FakeDB()
+        policy = unit_test_utils.FakePolicyEnforcer()
+        notifier = unit_test_utils.FakeNotifier()
+        store = unit_test_utils.FakeStoreAPI()
+        self.controller = glance.api.v2.image_data.ImageDataController()
+        self.controller.gateway = FakeGateway(db, store, notifier, policy,
+                                              self.image_repo)
+
+    def test_upload(self):
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage('abcd')
+        self.image_repo.result = image
+        self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', 4)
+        self.assertEqual('YYYY', image.data)
+        self.assertEqual(4, image.size)
+
+    def test_upload_invalid_backend_in_request_header(self):
+        request = unit_test_utils.get_fake_request()
+        request.headers['x-image-meta-store'] = 'dummy'
+        image = FakeImage('abcd')
+        self.image_repo.result = image
+        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.upload,
+                          request, unit_test_utils.UUID2, 'YYYY', 4)
